@@ -769,7 +769,7 @@ tcpreplay_set_tcpprep_cache(tcpreplay_t *ctx, char *file)
  * Enable verbose mode
  */
 int
-tcpreplay_set_verbose(tcpreplay_t *ctx, bool value)
+tcpreplay_set_verbose(tcpreplay_t *ctx, bool value _U_)
 {
     assert(ctx);
 #ifdef ENABLE_VERBOSE
@@ -788,7 +788,7 @@ tcpreplay_set_verbose(tcpreplay_t *ctx, bool value)
  * verbose mode.  See TCPDUMP_ARGS in tcpdump.h for the default options
  */
 int
-tcpreplay_set_tcpdump_args(tcpreplay_t *ctx, char *value)
+tcpreplay_set_tcpdump_args(tcpreplay_t *ctx, char *value _U_)
 {
     assert(ctx);
 #ifdef ENABLE_VERBOSE
@@ -808,7 +808,7 @@ tcpreplay_set_tcpdump_args(tcpreplay_t *ctx, char *value)
  * tcpdump lives
  */
 int
-tcpreplay_set_tcpdump(tcpreplay_t *ctx, tcpdump_t *value)
+tcpreplay_set_tcpdump(tcpreplay_t *ctx, tcpdump_t *value _U_)
 {
     assert(ctx);
 #ifdef ENABLE_VERBOSE
@@ -1158,6 +1158,11 @@ tcpreplay_replay(tcpreplay_t *ctx)
             if ((rcode = tcpr_replay_index(ctx)) < 0)
                 return rcode;
 
+            if (!ctx->abort && ctx->options->loopdelay_ms > 0) {
+                usleep(ctx->options->loopdelay_ms * 1000);
+                gettimeofday(&ctx->stats.end_time, NULL);
+            }
+
             if (ctx->options->stats == 0 && !ctx->abort)
                 packet_stats(&ctx->stats);
         }
